package operation

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/web"
	kit "shylinux.com/x/toolkits"

	"2025-dev.shylinux.com/x/20250215-cluster/src/travel"
	"shylinux.com/x/operation/src/operation/model"
)

type gateway struct {
	Table
	cluster       cluster
	travel        travel.Travel
	order         string `data:"2"`
	fields        string `data:"title,content,space,cluster_uid"`
	createProject string `name:"createProject name* repos binary" help:"创建" style:"notice" role:"worker"`
	scanProject   string `name:"scanProject" help:"扫描" role:"worker"`
	open          string `name:"open" role:"leader"`
	modify        string `name:"modify title content" style:"danger" role:"leader"`
}

func (s gateway) List(m *ice.Message, arg ...string) {
	if s.ValueList(m, arg); s.IsLeader(m) {
		m.PushAction(s.CreateProject, s.ScanProject, s.Open, s.Modify)
	} else if s.IsWorker(m) {
		m.PushAction(s.CreateProject, s.ScanProject)
	} else {
		m.PushAction()
	}
	if m.Action(); m.Length() == 0 {
		m.Echo("请到「云集群」购买主机")
	} else {
		s.SelectJoinUser(m).Display("")
	}
}
func (s gateway) CreateProject(m *ice.Message, arg ...string) {
	m.Option(web.SPACE, s.space(m))
	m.Cmd(project{}, s.Create, arg, model.GATEWAY_UID, m.Option(model.UID))
	m.ProcessRefresh()
}
func (s gateway) ScanProject(m *ice.Message, arg ...string) {
	m.Cmd(web.SPACE, s.space(m.Spawn()), web.DREAM).Table(func(value ice.Maps) {
		if value[mdb.TYPE] == web.WORKER {
			m.Cmd(project{}, s.InsertIfNeed, mdb.NAME, value[mdb.NAME], model.GATEWAY_UID, m.Option(model.UID), m.OptionSimple(model.CLOUD_UID))
		}
	})
	m.ProcessRefresh()
}
func (s gateway) Open(m *ice.Message, arg ...string) {
	m.ProcessOpen(m.Options(ice.MSG_USERPOD, "").MergePodCmd(s.space(m), web.ADMIN))
}
func (s gateway) Modify(m *ice.Message, arg ...string) {
	s.Update(m, arg, m.OptionSimple(model.CLOUD_UID, model.UID)...)
}

func init() { ice.TeamCtxCmd(gateway{}) }

func (s gateway) Payfor(m *ice.Message, arg ...string) {
	s.Transaction(m, func() {
		msg := m.Spawn()
		if msg = s.Orders(msg, "id").Limit(msg, 1).SelectForUpdate(msg, "(cloud_uid IS NULL OR cloud_uid = '') AND (cluster_uid = ?)", m.Option(model.CLUSTER_UID)); msg.Length() == 0 {
			m.Echo("主机不够用啦！请等待管理员添加。")
		} else {
			s.Update(m, m.OptionSimple(model.CLOUD_UID, model.USER_UID), msg.AppendSimple(model.UID)...)
		}
	})
}
func (s gateway) Scan(m *ice.Message, arg ...string) {
	m.Cmd(web.SPACE, m.Option(web.SPACE), s.travel, s.travel.Concurrent, "2025-dev.shylinux.com/x/20250211-service", "web.chat.dev.service.gateway").Table(func(value ice.Maps) {
		s.InsertIfNeed(m, web.SPACE, value[web.SPACE], model.CLUSTER_UID, m.Option(model.CLUSTER_UID))
	})
	s.Fields(m, "count(*) AS total")
	s.Select(m, model.CLUSTER_UID, m.Option(model.CLUSTER_UID))
	s.Fields(m, "count(*) AS stock")
	s.Select(m, "cluster_uid = ? AND (cloud_uid IS NULL OR cloud_uid = '')", m.Option(model.CLUSTER_UID))
	m.Info("hwat %v ", m.FormatMeta())
}
func (s gateway) space(m *ice.Message, arg ...string) string {
	s.Tables(m, s.cluster).Fields(m, s.Key(s, web.SPACE), s.KeyAS(s.cluster, web.SPACE))
	s.Select(m, kit.Simple(m.OptionSimple(model.CLOUD_UID), s.Key(s, model.UID), m.Option(model.UID))...)
	return kit.Keys(m.Append(model.CLUSTER_SPACE), m.Append(web.SPACE))
}
